import com.urbancode.anthill3.AHPTool
import com.urbancode.anthill3.plugin.jira.FailMode
import com.urbancode.anthill3.plugin.jira.JIRAHelper
import com.urbancode.anthill3.plugin.jira.XMLHelper
import org.apache.http.impl.client.CloseableHttpClient

final def ahptool = new AHPTool()

final def props = ahptool.getStepProperties()
final def serverUrl       = props['integration/serverUrl']
final def username        = props['integration/username']
final def password        = props['integration/passwordScript'] ?: props['integration/password']
final def issueKeyPattern = props['issueKeyPattern']
final def commentBody     = props['commentBody']
final def failMode        = FailMode.valueOf(props['failMode'])

//--------------------------------------------------------------------------------------------------
// Script content
//--------------------------------------------------------------------------------------------------

println "Server: $serverUrl"
println "Issue Key Pattern: $issueKeyPattern"
println "Comment: $commentBody"
println "Fail mode: $failMode"
println ""

def issueNotFoundExceptions = [];
def issueIdSet = [] as Set
JIRAHelper jira = new JIRAHelper()
CloseableHttpClient client

String body = '{"body":"' + commentBody + '"}'

try {
    XMLHelper helper = new XMLHelper(issueKeyPattern)
    helper.getChangesets()
    issueIdSet = helper.getIssueIdSet()

    issueIdSet.each { issueId ->
        client = jira.createClient(username, password)
        try {
            if (!jira.addIssueComment(client, serverUrl, issueId, body)) {
                issueNotFoundExceptions << "${issueId} failed to add comment"
                if (failMode == FailMode.FAIL_FAST) {
                    throw new IllegalStateException("Fail Fast: Add comment failed. " +
                        "Review error output for details.")
                }
            }
        }
        finally {
            client.close()
        }
        println "---------------------------------------"
    }
}
catch (Exception e) {
    e.printStackTrace()
    throw new Exception("Full execution of add comment failed.")
}

//--------------------------------------------------------------------------------------------------
// Check post conditions
//--------------------------------------------------------------------------------------------------
if (failMode == FailMode.FAIL_ON_NO_UPDATES && !issueIds) {
    throw new IllegalStateException("No issues found to update.");
}

if (issueNotFoundExceptions.size() > 0) {
    println("Failed to add comments to "+ issueNotFoundExceptions.size() + " issues");
    if (failMode != FailMode.WARN_ONLY) {
        // throw the first exception we found as our failure
        throw new IllegalStateException("Initial Error: " + issueNotFoundExceptions[0])
    }
 }
