import groovy.json.JsonBuilder
import com.urbancode.anthill3.AHPTool;
import com.urbancode.anthill3.plugin.jira.JIRAHelper
import org.apache.http.impl.client.CloseableHttpClient

final def ahptool = new AHPTool()
final def props = ahptool.getStepProperties()

final def serverUrl        = props['integration/serverUrl']
final def username         = props['integration/username']
final def password         = props['integration/passwordScript'] ?: props['integration/password']

final def projectKey       = props['projectKey']
final def assignee         = props['assignee']
final def summary          = props['summary']
final def environment      = props['environment']
final def issueDescription = props['issueDescription']
final def priorityName     = props['priorityName']
final def issueTypeName    = props['issueTypeName']

// Constants
final def dueDate          = Calendar.getInstance()

// Construct an empty JIRAHelper class to call helper functions
JIRAHelper jira            = new JIRAHelper()

//------------------------------------------------------------------------------
// Script content
//------------------------------------------------------------------------------

println "Server: $serverUrl"
println "Project: $projectKey"
println "Assignee: $assignee"
println "Priority: $priorityName"
println "Issue Type: $issueTypeName"
println ""


// Construct the HTTP client to allow user authentication
CloseableHttpClient client = jira.createClient(username, password)
try {
    // Create Issue as a HashMap object
    def fieldMap = [:]
    fieldMap.put("project", ["key": projectKey])
    fieldMap.put("issuetype", ["name": issueTypeName])
    fieldMap.put("summary", summary)
    fieldMap.put("assignee", ["name": assignee])
    fieldMap.put("duedate", dueDate)
    // Optional fields
    if (priorityName) {
        fieldMap.put("priority", ["name": priorityName])
    }
    if (environment) {
        fieldMap.put("environment", environment)
    }
    if (issueDescription) {
        fieldMap.put("description", issueDescription)
    }

    def issueMap = [:]
    issueMap.put("fields", fieldMap)

    // Change mapping to JSON and prepare it to be sent via REST call
    JsonBuilder issueJSON = new JsonBuilder(issueMap)

    if (!jira.createIssue(client, serverUrl, issueJSON)) {
        println "Create Issue failed. Review error output for details."
        System.exit(1)
    }
}
finally {
    client.close()
}