import groovy.xml.StreamingMarkupBuilder
import org.apache.http.impl.client.CloseableHttpClient
import com.urbancode.anthill3.AHPTool
import com.urbancode.anthill3.plugin.jira.JIRAHelper
import com.urbancode.anthill3.plugin.jira.XMLHelper

final def ahptool = new AHPTool()
final def props = ahptool.getStepProperties()

final def serverUrl       = props['integration/serverUrl']
final def username        = props['integration/username']
final def password        = props['integration/passwordScript'] ?: props['integration/password']
final def issueKeyPattern = props['issueKeyPattern']

//------------------------------------------------------------------------------
// Script content
//------------------------------------------------------------------------------

println "Server: $serverUrl"
println "Issue Key Pattern: $issueKeyPattern"
println ""

JIRAHelper jira = new JIRAHelper()
CloseableHttpClient client = jira.createClient(username, password)

try {
    XMLHelper helper = new XMLHelper(issueKeyPattern)
    helper.getChangesets()
    def issueIdSet = helper.getIssueIdSet()
    def issueIdToChangeSetsMap = helper.getIssueIdToChangeSetListMap()
    def jiraIssues = []
    issueIdSet.each { issueKey ->
        def jiraIssue = jira.getIssue(client, serverUrl, issueKey)
        if (!jiraIssue) {
            println("\tSkipping Issue " + issueKey + ": Specified Issue not found.")
        }
        else {
            jiraIssues << jiraIssue
        }
    }

    def builder = new StreamingMarkupBuilder()
    builder.encoding = 'UTF-8'
    def issuesXml = builder.bind {
        'issues'() {
            jiraIssues.each { jiraIssue ->
                def issueKey = jiraIssue.key
                issueIdToChangeSetsMap.get(issueKey).each { changeId ->
                    'issue'("id":issueKey, "issue-tracker":"JIRA", "change-id":changeId) {
                        'name'(jiraIssue.fields?.summary)
                        'description'(jiraIssue.fields?.description)
                        'type'(jiraIssue.fields?.issuetype?.name)
                        'owner'(jiraIssue.fields?.assignee?.name)
                        'status'(jiraIssue.fields?.status?.name)
                        'url'(serverUrl + '/browse/' + issueKey)
                    }
                }
            }
        }
    }
    helper.sendPost(issuesXml.toString())
}
finally {
    client.close()
}
